@@ -7,9 +7,23 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2012 jointly by the following organizations: 
+ * Copyright (C) 2020 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
- *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
+ *     2. University of Heidelberg, Heidelberg, Germany
+ *     3. University College London, London, UK
+ *
+ * Copyright (C) 2019 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. University of Heidelberg, Heidelberg, Germany
+ *
+ * Copyright (C) 2013-2018 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *     3. University of Heidelberg, Heidelberg, Germany
+ *
+ * Copyright (C) 2009-2013 jointly by the following organizations: 
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *  
  * Copyright (C) 2006-2008 by the California Institute of Technology,
  *     Pasadena, CA, USA 
@@ -37,7 +51,7 @@
  *
  * A version string of the form "1.2.3".
  */
-#define LIBSBML_DOTTED_VERSION	"5.4.1"
+#define LIBSBML_DOTTED_VERSION	"5.21.0"
 
 
 /**
@@ -48,7 +62,7 @@
  * libSBML is released, making it easy to use less-than and greater-than
  * comparisons when testing versions numbers.
  */
-#define LIBSBML_VERSION		50401
+#define LIBSBML_VERSION		52100
 
 
 /**
@@ -56,7 +70,7 @@
  *
  * The numeric version as a string: version 1.2.3 becomes "10203".
  */
-#define LIBSBML_VERSION_STRING	"50401"
+#define LIBSBML_VERSION_STRING	"52100"
 
 
 LIBSBML_CPP_NAMESPACE_BEGIN
@@ -98,6 +112,42 @@ const char*
 getLibSBMLVersionString ();
 
 
+/**
+ * Returns an indication whether libSBML has been compiled with
+ * against a specific library. 
+ *
+ * @param option the library to test against, this can be one of
+ *        "expat", "libxml", "xerces-c", "bzip2", "zip"
+ * 
+ * @return 0 in case the libSBML has not been compiled against 
+ *         that library and nonzero otherwise (for libraries 
+ *         that define an integer version number that number will 
+ *         be returned).
+ *
+ * @see getLibSBMLDependencyVersionOf(const char* option)
+ */
+LIBSBML_EXTERN
+int 
+isLibSBMLCompiledWith(const char* option);
+
+
+/**
+ * Returns the version string for the dependency library used. 
+ *
+ * @param option the library for which the version
+ *        should be retrieved, this can be one of
+ *        "expat", "libxml", "xerces-c", "bzip2", "zip"
+ * 
+ * @return NULL in case libSBML has not been compiled against 
+ *         that library and a version string otherwise.
+ *
+ * @see isLibSBMLCompiledWith(const char* option)
+ */
+LIBSBML_EXTERN
+const char* 
+getLibSBMLDependencyVersionOf(const char* option);
+
+
 END_C_DECLS
 LIBSBML_CPP_NAMESPACE_END
 
