@@ -1,53 +1,41 @@
 /**
- * Filename    : BoundingBox.cpp
- * Description : SBML Layout BoundingBox source
- * Organization: European Media Laboratories Research gGmbH
- * Created     : 2004-07-15
+ * @file    BoundingBox.cpp
+ * @brief   Implementation of BoundingBox for SBML Layout.
+ * @author  Ralph Gauges
+ * 
+ * <!--------------------------------------------------------------------------
+ * This file is part of libSBML.  Please visit http://sbml.org for more
+ * information about SBML, and the latest version of libSBML.
  *
- * Copyright 2004 European Media Laboratories Research gGmbH
+ * Copyright (C) 2020 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. University of Heidelberg, Heidelberg, Germany
+ *     3. University College London, London, UK
  *
+ * Copyright (C) 2019 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. University of Heidelberg, Heidelberg, Germany
+ *
+ * Copyright (C) 2013-2018 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *     3. University of Heidelberg, Heidelberg, Germany
+ * 
+ * Copyright (C) 2009-2013 jointly by the following organizations: 
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *  
+ * Copyright (C) 2004-2008 by European Media Laboratories Research gGmbH,
+ *     Heidelberg, Germany
+ * 
  * This library is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published
- * by the Free Software Foundation; either version 2.1 of the License, or
- * any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
- * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
- * documentation provided hereunder is on an "as is" basis, and the
- * European Media Laboratories Research gGmbH have no obligations to
- * provide maintenance, support, updates, enhancements or modifications.
- * In no event shall the European Media Laboratories Research gGmbH be
- * liable to any party for direct, indirect, special, incidental or
- * consequential damages, including lost profits, arising out of the use of
- * this software and its documentation, even if the European Media
- * Laboratories Research gGmbH have been advised of the possibility of such
- * damage.  See the GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * The original code contained here was initially developed by:
- *
- *     Ralph Gauges
- *     Bioinformatics Group
- *     European Media Laboratories Research gGmbH
- *     Schloss-Wolfsbrunnenweg 31c
- *     69118 Heidelberg
- *     Germany
- *
- *     http://www.eml-research.de/english/Research/BCB/
- *     mailto:ralph.gauges@eml-r.villa-bosch.de
- *
- * Contributor(s):
- *
- *     Akiya Jouraku <jouraku@bio.keio.ac.jp>
- *     Modified this file for package extension in libSBML5
- *
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.  A copy of the license agreement is provided
+ * in the file named "LICENSE.txt" included with this software distribution
+ * and also available online as http://sbml.org/software/libsbml/license.html
+ * ------------------------------------------------------------------------ -->
  */
 
-
 #include <sbml/packages/layout/sbml/BoundingBox.h>
 #include <sbml/packages/layout/util/LayoutUtilities.h>
 
@@ -58,11 +46,30 @@
 #include <sbml/xml/XMLInputStream.h>
 #include <sbml/xml/XMLOutputStream.h>
 
+#include <sbml/util/ElementFilter.h>
+
 #include <sbml/packages/layout/extension/LayoutExtension.h>
+#include <sbml/packages/layout/validator/LayoutSBMLError.h>
 
 LIBSBML_CPP_NAMESPACE_BEGIN
+#ifdef __cplusplus
 
-/**
+
+List*
+BoundingBox::getAllElements(ElementFilter *filter)
+{
+  List* ret = new List();
+  List* sublist = NULL;
+
+  ADD_FILTERED_ELEMENT(ret, sublist, mPosition, filter);  
+  ADD_FILTERED_ELEMENT(ret, sublist, mDimensions, filter);  
+
+  ADD_FILTERED_FROM_PLUGIN(ret, sublist, filter);
+
+  return ret;
+}
+
+/*
  * Default Constructor set position and dimensions to (0.0,0.0,0.0) and the
  * id to an empty string.
  */ 
@@ -70,6 +77,8 @@ BoundingBox::BoundingBox(unsigned int level, unsigned int version, unsigned int
  : SBase(level,version)
   ,mPosition(level,version,pkgVersion)
   ,mDimensions(level,version,pkgVersion)
+  ,mPositionExplicitlySet (false)
+  ,mDimensionsExplicitlySet (false)
 {
   mPosition.setElementName("position");
   setSBMLNamespacesAndOwn(new LayoutPkgNamespaces(level,version,pkgVersion));  
@@ -81,6 +90,8 @@ BoundingBox::BoundingBox(LayoutPkgNamespaces* layoutns)
  : SBase(layoutns)
   ,mPosition(layoutns)
   ,mDimensions(layoutns)
+  ,mPositionExplicitlySet (false)
+  ,mDimensionsExplicitlySet (false)
 {
   //
   // set the element namespace of this object
@@ -98,7 +109,7 @@ BoundingBox::BoundingBox(LayoutPkgNamespaces* layoutns)
 }
 
 
-/**
+/*
  * Copy constructor.
  */
 BoundingBox::BoundingBox(const BoundingBox& orig):SBase(orig)
@@ -106,12 +117,14 @@ BoundingBox::BoundingBox(const BoundingBox& orig):SBase(orig)
   this->mId = orig.mId;
   this->mPosition=orig.mPosition;
   this->mDimensions=orig.mDimensions;
+  this->mPositionExplicitlySet = orig.mPositionExplicitlySet;
+  this->mDimensionsExplicitlySet = orig.mDimensionsExplicitlySet;
 
   connectToChild();
 }
 
 
-/**
+/*
  * Assignment operator
  */
 BoundingBox& BoundingBox::operator=(const BoundingBox& orig)
@@ -123,6 +136,8 @@ BoundingBox& BoundingBox::operator=(const BoundingBox& orig)
     this->mId = orig.mId;
     this->mPosition=orig.mPosition;
     this->mDimensions=orig.mDimensions;
+    this->mPositionExplicitlySet = orig.mPositionExplicitlySet;
+    this->mDimensionsExplicitlySet = orig.mDimensionsExplicitlySet;
 
     connectToChild();
   }
@@ -131,20 +146,25 @@ BoundingBox& BoundingBox::operator=(const BoundingBox& orig)
 }
 
 
-/**
+/*
  * Constructor set position and dimensions to (0.0,0.0,0.0) and the id to a
  * copy of the given string.
  */ 
 BoundingBox::BoundingBox (LayoutPkgNamespaces* layoutns, const std::string id)
  : 
   SBase(layoutns)
- ,mId (id)
+// ,mId (id)
  ,mPosition(layoutns)
  ,mDimensions(layoutns)
+  ,mPositionExplicitlySet (false)
+  ,mDimensionsExplicitlySet (false)
 {
+  setId(id);
+
   //
   // set the element namespace of this object
   //
+
   setElementNamespace(layoutns->getURI());
 
   mPosition.setElementName("position");
@@ -154,17 +174,21 @@ BoundingBox::BoundingBox (LayoutPkgNamespaces* layoutns, const std::string id)
 }
 
 
-/**
+/*
  * Constructor which sets the id, the coordinates and the dimensions to the
  * given 2D values.
  */ 
 BoundingBox::BoundingBox (LayoutPkgNamespaces* layoutns, const std::string id,
                           double x, double y, double width, double height)
   : SBase     (layoutns)
-  , mId (id)
+//  , mId (id)
   , mPosition  (layoutns, x, y, 0.0)
   , mDimensions(layoutns, width, height, 0.0)
+  ,mPositionExplicitlySet (true)
+  ,mDimensionsExplicitlySet (true)
 {
+  setId(id);
+
   //
   // set the element namespace of this object
   //
@@ -180,7 +204,7 @@ BoundingBox::BoundingBox (LayoutPkgNamespaces* layoutns, const std::string id,
 }
 
 
-/**
+/*
  * Constructor which sets the id, the coordinates and the dimensions to the
  * given 3D values.
  */ 
@@ -188,10 +212,14 @@ BoundingBox::BoundingBox (LayoutPkgNamespaces* layoutns, const std::string id,
                           double x, double y, double z,
                           double width, double height, double depth)
   : SBase     (layoutns)
-  , mId (id)
+//  , mId (id)
   , mPosition  (layoutns, x, y, z)
   , mDimensions(layoutns, width, height, depth)
+  ,mPositionExplicitlySet (true)
+  ,mDimensionsExplicitlySet (true)
 {
+  setId(id);
+
   //
   // set the element namespace of this object
   //
@@ -207,7 +235,7 @@ BoundingBox::BoundingBox (LayoutPkgNamespaces* layoutns, const std::string id,
 }
 
         
-/**
+/*
  * Constructor which sets the id, the coordinates and the dimensions to the
  * given 3D values.
  */ 
@@ -215,20 +243,27 @@ BoundingBox::BoundingBox (LayoutPkgNamespaces* layoutns, const std::string id,
                           const Point*      p,
                           const Dimensions* d)
   : SBase     (layoutns)
-  , mId (id)
+//  , mId (id)
   , mPosition(layoutns)
   , mDimensions(layoutns)
+  ,mPositionExplicitlySet (true)
+  ,mDimensionsExplicitlySet (true)
 {
+  setId(id);
+
   //
   // set the element namespace of this object
   //
   setElementNamespace(layoutns->getURI());
 
-  mPosition.setElementName("position");
   if(p)
   {
       this->mPosition=*p;   
   }
+
+  mPosition.setElementName("position");
+
+
   if(d)
   {
       this->mDimensions=*d;   
@@ -241,14 +276,16 @@ BoundingBox::BoundingBox (LayoutPkgNamespaces* layoutns, const std::string id,
   loadPlugins(layoutns);
 }
 
-/**
+/*
  * Creates a new BoundingBox from the given XMLNode
  */
 BoundingBox::BoundingBox(const XMLNode& node, unsigned int l2version)
  :  SBase(2,l2version)
-  , mId("")
+//  , mId("")
   , mPosition(2,l2version)
   , mDimensions(2,l2version)
+  ,mPositionExplicitlySet (false)
+  ,mDimensionsExplicitlySet (false)
 {
     mPosition.setElementName("position");
 
@@ -267,10 +304,12 @@ BoundingBox::BoundingBox(const XMLNode& node, unsigned int l2version)
         if(childName=="position")
         {
             this->mPosition=Point(*child);
+            this->mPositionExplicitlySet = true;
         }
         else if(childName=="dimensions")
         {
             this->mDimensions=Dimensions(*child);
+            this->mDimensionsExplicitlySet = true;
         }
         else if(childName=="annotation")
         {
@@ -291,7 +330,7 @@ BoundingBox::BoundingBox(const XMLNode& node, unsigned int l2version)
 }
 
 
-/**
+/*
  * Destructor which does nothing.
  */ 
 BoundingBox::~BoundingBox ()
@@ -299,7 +338,7 @@ BoundingBox::~BoundingBox ()
 }
 
 
-/**
+/*
  * Does nothing since no defaults are defined for a BundingBox.
  */ 
 void BoundingBox::initDefaults ()
@@ -307,7 +346,7 @@ void BoundingBox::initDefaults ()
 }
 
 
-/**
+/*
   * Returns the value of the "id" attribute of this BoundingBox.
   */
 const std::string& BoundingBox::getId () const
@@ -316,7 +355,7 @@ const std::string& BoundingBox::getId () const
 }
 
 
-/**
+/*
   * Predicate returning @c true or @c false depending on whether this
   * BoundingBox's "id" attribute has been set.
   */
@@ -325,7 +364,7 @@ bool BoundingBox::isSetId () const
   return (mId.empty() == false);
 }
 
-/**
+/*
   * Sets the value of the "id" attribute of this BoundingBox.
   */
 int BoundingBox::setId (const std::string& id)
@@ -334,7 +373,7 @@ int BoundingBox::setId (const std::string& id)
 }
 
 
-/**
+/*
   * Unsets the value of the "id" attribute of this BoundingBox.
   */
 int BoundingBox::unsetId ()
@@ -351,8 +390,8 @@ int BoundingBox::unsetId ()
 }
 
 
-/**
- * Returns the position of the BoundingBox as const referece to a Point
+/*
+ * Returns the position of the BoundingBox as const reference to a Point
  * object.
  */ 
 const Point*
@@ -362,8 +401,8 @@ BoundingBox::getPosition () const
 }
 
 
-/**
- * Returns the dimensions of the BoundingBox as const referece to a
+/*
+ * Returns the dimensions of the BoundingBox as const reference to a
  * Dimensions object.
  */ 
 const Dimensions*
@@ -373,8 +412,8 @@ BoundingBox::getDimensions () const
 }
 
 
-/**
- * Returns the position of the BoundingBox as referece to a Point object.
+/*
+ * Returns the position of the BoundingBox as reference to a Point object.
  */ 
 Point*
 BoundingBox::getPosition ()
@@ -383,8 +422,8 @@ BoundingBox::getPosition ()
 }
 
 
-/**
- * Returns the dimensions of the BoundingBox as referece to a Dimensions
+/*
+ * Returns the dimensions of the BoundingBox as reference to a Dimensions
  * object.
  */ 
 Dimensions*
@@ -394,19 +433,20 @@ BoundingBox::getDimensions ()
 }
 
 
-/**
+/*
  * Sets the position to a copy of the Point object given.
  */ 
 void BoundingBox::setPosition (const Point* p)
 {
     if(!p) return;  
     this->mPosition = Point(*p);
-	this->mPosition.setElementName("position");
+  this->mPosition.setElementName("position");
     this->mPosition.connectToParent(this);
+    this->mPositionExplicitlySet = true;
 }
 
 
-/**
+/*
  * Sets the dimensions to a copy of the Dimensions object given.
  */ 
 void
@@ -415,10 +455,23 @@ BoundingBox::setDimensions (const Dimensions* d)
   if(!d) return;
   this->mDimensions = Dimensions(*d);
   this->mDimensions.connectToParent(this);
+  this->mDimensionsExplicitlySet = true;
+}
+
+bool
+BoundingBox::getPositionExplicitlySet() const
+{
+  return mPositionExplicitlySet;
+}
+
+bool
+BoundingBox::getDimensionsExplicitlySet() const
+{
+  return mDimensionsExplicitlySet;
 }
 
 
-/**
+/*
  * Sets the x offset of the BoundingBox.
  */
 void
@@ -428,7 +481,7 @@ BoundingBox::setX(double x)
 }
 
 
-/**
+/*
  * Sets the y offset of the BoundingBox.
  */
 void
@@ -438,7 +491,7 @@ BoundingBox::setY(double y)
 }
 
 
-/**
+/*
  * Sets the z offset of the BoundingBox.
  */
 void
@@ -448,7 +501,7 @@ BoundingBox::setZ(double z)
 }
 
 
-/**
+/*
  * Sets the width of the BoundingBox.
  */
 void
@@ -458,7 +511,7 @@ BoundingBox::setWidth(double width)
 }
 
 
-/**
+/*
  * Sets the height of the BoundingBox.
  */
 void
@@ -468,7 +521,7 @@ BoundingBox::setHeight(double height)
 }
 
 
-/**
+/*
  * Sets the depth of the BoundingBox.
  */
 void
@@ -477,7 +530,7 @@ BoundingBox::setDepth(double depth)
   this->mDimensions.setDepth(depth);
 }
 
-/**
+/*
  * Returns the x offset of the bounding box.
  */
 double
@@ -486,7 +539,7 @@ BoundingBox::x() const
   return this->mPosition.x();
 }
 
-/**
+/*
  * Returns the y offset of the bounding box.
  */
 double
@@ -495,7 +548,7 @@ BoundingBox::y() const
   return this->mPosition.y();
 }
 
-/**
+/*
  * Returns the z offset of the bounding box.
  */
 double
@@ -504,7 +557,7 @@ BoundingBox::z() const
   return this->mPosition.z();
 }
 
-/**
+/*
  * Returns the width of the bounding box.
  */
 double
@@ -513,7 +566,7 @@ BoundingBox::width() const
   return this->mDimensions.width();
 }
 
-/**
+/*
  * Returns the height of the bounding box.
  */
 double
@@ -522,7 +575,7 @@ BoundingBox::height() const
   return this->mDimensions.height();
 }
 
-/**
+/*
  * Returns the depth of the bounding box.
  */
 double
@@ -531,13 +584,14 @@ BoundingBox::depth() const
   return this->mDimensions.depth();
 }
 
-/**
+/** @cond doxygenLibsbmlInternal */
+/*
  * Subclasses should override this method to write out their contained
- * SBML objects as XML elements.  Be sure to call your parents
+ * SBML objects as XML elements.  Be sure to call your parent's
  * implementation of this method as well.  For example:
  *
  *   SBase::writeElements(stream);
- *   mReactans.write(stream);
+ *   mReactants.write(stream);
  *   mProducts.write(stream);
  *   ...
  */
@@ -552,9 +606,10 @@ void BoundingBox::writeElements (XMLOutputStream& stream) const
   //
   SBase::writeExtensionElements(stream);
 }
+/** @endcond */
 
-/**
- * Subclasses should override this method to return XML element name of
+/*
+ * Returns the XML element name of
  * this SBML object.
  */
 const std::string& BoundingBox::getElementName () const 
@@ -563,8 +618,8 @@ const std::string& BoundingBox::getElementName () const
   return name;
 }
 
-/**
- * @return a (deep) copy of this Model.
+/*
+ * @return a (deep) copy of this BoundingBox.
  */
 BoundingBox* 
 BoundingBox::clone () const
@@ -573,10 +628,7 @@ BoundingBox::clone () const
 }
 
 
-/**
- * @return the SBML object corresponding to next XMLToken in the
- * XMLInputStream or NULL if the token was not recognized.
- */
+/** @cond doxygenLibsbmlInternal */
 SBase*
 BoundingBox::createObject (XMLInputStream& stream)
 {
@@ -585,23 +637,31 @@ BoundingBox::createObject (XMLInputStream& stream)
 
   if (name == "dimensions")
   {
+    if (getDimensionsExplicitlySet() == true)
+    {
+      getErrorLog()->logPackageError("layout", LayoutBBoxAllowedElements, 
+          getPackageVersion(), getLevel(), getVersion(), "", getLine(), getColumn());
+    }
     object = &mDimensions;
+    mDimensionsExplicitlySet = true;
   }
 
   else if ( name == "position"    )
   {
+    if (getPositionExplicitlySet() == true)
+    {
+      getErrorLog()->logPackageError("layout", LayoutBBoxAllowedElements, 
+          getPackageVersion(), getLevel(), getVersion(), "", getLine(), getColumn());
+    }
       object = &mPosition;
+      mPositionExplicitlySet = true;
   }
 
   return object;
 }
+/** @endcond */
 
-/**
- * Subclasses should override this method to get the list of
- * expected attributes.
- * This function is invoked from corresponding readAttributes()
- * function.
- */
+/** @cond doxygenLibsbmlInternal */
 void
 BoundingBox::addExpectedAttributes(ExpectedAttributes& attributes)
 {
@@ -609,40 +669,70 @@ BoundingBox::addExpectedAttributes(ExpectedAttributes& attributes)
 
   attributes.add("id");
 }
+/** @endcond */
 
-
-/**
- * Subclasses should override this method to read values from the given
- * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
- */
-
+/** @cond doxygenLibsbmlInternal */
 void BoundingBox::readAttributes (const XMLAttributes& attributes,
                                   const ExpectedAttributes& expectedAttributes)
 {
-  SBase::readAttributes(attributes,expectedAttributes);
-
   const unsigned int sbmlLevel   = getLevel  ();
   const unsigned int sbmlVersion = getVersion();
 
-  bool assigned = attributes.readInto("id", mId, getErrorLog(), false, getLine(), getColumn());
-  if (assigned && mId.empty())
-  {
-    logEmptyString(mId, sbmlLevel, sbmlVersion, "<boundingBox>");
-  }
-  if (!SyntaxChecker::isValidInternalSId(mId)) logError(InvalidIdSyntax);
-}
+  unsigned int numErrs;
 
-/**
- * Subclasses should override this method to write their XML attributes
- * to the XMLOutputStream.  Be sure to call your parents implementation
- * of this method as well.  For example:
- *
- *   SBase::writeAttributes(stream);
- *   stream.writeAttribute( "id"  , mId   );
- *   stream.writeAttribute( "name", mName );
- *   ...
- */
+  SBase::readAttributes(attributes, expectedAttributes);
+
+  // look to see whether an unknown attribute error was logged
+  if (getErrorLog() != NULL)
+  {
+    numErrs = getErrorLog()->getNumErrors();
+    for (int n = (int)numErrs-1; n >= 0; n--)
+    {
+      if (getErrorLog()->getError((unsigned int)n)->getErrorId() == UnknownPackageAttribute)
+      {
+        const std::string details =
+          getErrorLog()->getError((unsigned int)n)->getMessage();
+        getErrorLog()->remove(UnknownPackageAttribute);
+        getErrorLog()->logPackageError("layout", LayoutBBoxAllowedAttributes,
+          getPackageVersion(), sbmlLevel, sbmlVersion, details, getLine(), getColumn());
+      }
+      else if (getErrorLog()->getError((unsigned int)n)->getErrorId() == UnknownCoreAttribute)
+      {
+        const std::string details =
+          getErrorLog()->getError((unsigned int)n)->getMessage();
+        getErrorLog()->remove(UnknownCoreAttribute);
+        getErrorLog()->logPackageError("layout", 
+          LayoutBBoxAllowedCoreAttributes,
+          getPackageVersion(), sbmlLevel, sbmlVersion, details, getLine(), getColumn());
+      }
+    }
+  }
+
+  bool assigned = false;
+
+  //
+  // id SId  ( use = "optional" )
+  //
+  assigned = attributes.readInto("id", mId);
+
+  if (assigned == true && getErrorLog() != NULL)
+  {
+    // check string is not empty and correct syntax
+
+    if (mId.empty() == true)
+    {
+      logEmptyString(mId, getLevel(), getVersion(), "<BoundingBox>");
+    }
+    else if (SyntaxChecker::isValidSBMLSId(mId) == false)
+    {
+      getErrorLog()->logPackageError("layout", LayoutSIdSyntax, 
+        getPackageVersion(), sbmlLevel, sbmlVersion, "", getLine(), getColumn());
+    }
+  }
+}
+/** @endcond */
+
+/** @cond doxygenLibsbmlInternal */
 void BoundingBox::writeAttributes (XMLOutputStream& stream) const
 {
   SBase::writeAttributes(stream);
@@ -653,9 +743,10 @@ void BoundingBox::writeAttributes (XMLOutputStream& stream) const
   //
   SBase::writeExtensionAttributes(stream);
 }
+/** @endcond */
 
 
-/**
+/*
  * Returns the package type code  for this object.
  */
 int
@@ -665,47 +756,31 @@ BoundingBox::getTypeCode () const
 }
 
 
-/**
- * Accepts the given SBMLVisitor.
- */
+/** @cond doxygenLibsbmlInternal */
 bool
 BoundingBox::accept (SBMLVisitor& v) const
 {
-  /*
-  bool result=v.visit(*this);
+  v.visit(*this);
+
   mPosition.accept(v);
   mDimensions.accept(v);
-  v.leave(*this);*/
-  return false;
+  
+  v.leave(*this);
+  
+  return true;
 }
+/** @endcond */
 
-/**
+/*
  * Creates an XMLNode object from this.
  */
 XMLNode BoundingBox::toXML() const
 {
-  XMLNamespaces xmlns = XMLNamespaces();
-  XMLTriple triple = XMLTriple("boundingBox", "", "");
-  XMLAttributes att = XMLAttributes();
-  // add the SBase Ids
-  addSBaseAttributes(*this,att);
-  if(this->isSetId())
-  {
-    att.add("id",this->mId);
-  }
-  XMLToken token = XMLToken(triple, att, xmlns); 
-  XMLNode node(token);
-  // add the notes and annotations
-  if(this->mNotes) node.addChild(*this->mNotes);
-  if(this->mAnnotation) node.addChild(*this->mAnnotation);
-  // add position
-  node.addChild(this->mPosition.toXML("position"));
-  // add dimensions
-  node.addChild(this->mDimensions.toXML());
-  return node;
+  return getXmlNodeForSBase(this);
 }
 
 
+/** @cond doxygenLibsbmlInternal */
 /*
  * Sets the parent SBMLDocument of this SBML object.
  */
@@ -717,8 +792,10 @@ BoundingBox::setSBMLDocument (SBMLDocument* d)
   mPosition.setSBMLDocument(d);
   mDimensions.setSBMLDocument(d);
 }
+/** @endcond */
 
 
+/** @cond doxygenLibsbmlInternal */
 /*
  * Sets this SBML object to child SBML objects (if any).
  * (Creates a child-parent relationship by the parent)
@@ -726,11 +803,14 @@ BoundingBox::setSBMLDocument (SBMLDocument* d)
 void
 BoundingBox::connectToChild()
 {
+  SBase::connectToChild();
   mPosition.connectToParent(this);
   mDimensions.connectToParent(this);
 }
+/** @endcond */
 
-/**
+/** @cond doxygenLibsbmlInternal */
+/*
  * Enables/Disables the given package with this element and child
  * elements (if any).
  * (This is an internal implementation for enablePakcage function)
@@ -744,13 +824,11 @@ BoundingBox::enablePackageInternal(const std::string& pkgURI,
   mPosition.enablePackageInternal(pkgURI,pkgPrefix,flag);
   mDimensions.enablePackageInternal(pkgURI,pkgPrefix,flag);
 }
+/** @endcond */
 
 
-/**
- * Function that creates a BoundingBox_t object with position set to
- * (0.0,0.0,0.0) and dimensions set to (0.0,0.0,0.0). The id is set to the
- * empty string.
- */
+#endif /* __cplusplus */
+/** @cond doxygenIgnored */
 LIBSBML_EXTERN
 BoundingBox_t *
 BoundingBox_create (void)
@@ -759,11 +837,6 @@ BoundingBox_create (void)
 }
 
 
-/**
- * Function that creates a BoundingBox_t object with position set to
- * (0.0,0.0,0.0) and dimensions set to (0.0,0.0,0.0).  The id is set to the
- * given string.
- */
 LIBSBML_EXTERN
 BoundingBox_t *
 BoundingBox_createWith (const char *id)
@@ -773,10 +846,6 @@ BoundingBox_createWith (const char *id)
 }
 
 
-/**
- * Function that creates a BoundingBox_t object with the coordinates and
- * sizes given as arguments. The id is set to the empty string.
- */ 
 LIBSBML_EXTERN
 BoundingBox_t *
 BoundingBox_createWithCoordinates (const char *id,
@@ -788,9 +857,6 @@ BoundingBox_createWithCoordinates (const char *id,
 }
 
 
-/**
- * Frees all memory taken by the given BoundingBox_t object.
- */ 
 LIBSBML_EXTERN
 void
 BoundingBox_free (BoundingBox_t *bb)
@@ -799,234 +865,190 @@ BoundingBox_free (BoundingBox_t *bb)
 }
 
 
-/**
- * Does nothing since no defaults are defined for BoundingBox.
- */
 LIBSBML_EXTERN
 void
 BoundingBox_initDefaults (BoundingBox_t *bb)
 {
+  if (bb == NULL) return;
   bb->initDefaults();
 }
 
 
-/**
- * Returns the position as a Point_t object.
- */ 
 LIBSBML_EXTERN
 Point_t *
 BoundingBox_getPosition (BoundingBox_t *bb)
 {
+  if (bb == NULL) return NULL;
   return bb->getPosition();
 }
 
 
-/**
- * Returns the dimensions as a Dimensions_t object.
- */ 
 LIBSBML_EXTERN
 Dimensions_t *
 BoundingBox_getDimensions (BoundingBox_t *bb)
 {
+  if (bb == NULL) return NULL;
   return bb->getDimensions();
 }
 
 
-/**
- * Sets the position to a copy of the Point_t object given as argument.
- */
 LIBSBML_EXTERN
 void
 BoundingBox_setPosition (BoundingBox_t *bb, const Point_t *p)
 {
-   bb->setPosition(p);
+  if (bb == NULL) return;
+  bb->setPosition(p);
 }
 
 
-/**
- * Sets the dimensions to a copy of the Dimensions_t object given.
- */ 
 LIBSBML_EXTERN
 void
 BoundingBox_setDimensions (BoundingBox_t *bb, const Dimensions_t *d)
 {
+  if (bb == NULL) return;
   bb->setDimensions(d);
 }
 
-/**
- * Sets the x offset of the bounding box.
- */
 LIBSBML_EXTERN
 void
 BoundingBox_setX(BoundingBox_t* bb,double x)
 {
-    bb->setX(x);
+  if (bb == NULL) return;
+  bb->setX(x);
 }
 
-/**
- * Sets the y offset of the bounding box.
- */
 LIBSBML_EXTERN
 void
 BoundingBox_setY(BoundingBox_t* bb,double y)
 {
-    bb->setY(y);
+  if (bb == NULL) return;
+  bb->setY(y);
 }
 
 
-/**
- * Sets the z offset of the bounding box.
- */
 LIBSBML_EXTERN
 void
 BoundingBox_setZ(BoundingBox_t* bb,double z)
 {
-    bb->setZ(z);
+  if (bb == NULL) return;
+  bb->setZ(z);
 }
 
 
-/**
- * Sets the width of the bounding box.
- */
 LIBSBML_EXTERN
 void
 BoundingBox_setWidth(BoundingBox_t* bb,double width)
 {
-    bb->setWidth(width);
+  if (bb == NULL) return;
+  bb->setWidth(width);
 }
 
 
-/**
- * Sets the height of the bounding box.
- */
 LIBSBML_EXTERN
 void
 BoundingBox_setHeight(BoundingBox_t* bb,double height)
 {
-    bb->setHeight(height);
+  if (bb == NULL) return;
+  bb->setHeight(height);
 }
 
 
-/**
- * Sets the depth of the bounding box.
- */
 LIBSBML_EXTERN
 void
 BoundingBox_setDepth(BoundingBox_t* bb,double depth)
 {
-    bb->setDepth(depth);
+  if (bb == NULL) return;
+  bb->setDepth(depth);
 }
 
-/**
- * Returns the x offset of the bounding box
- */
 LIBSBML_EXTERN
 double
 BoundingBox_x(BoundingBox_t* bb)
 {
-    return bb->x();
+  if (bb == NULL) return std::numeric_limits<double>::quiet_NaN();
+  return bb->x();
 }
 
-/**
- * Returns the y offset of the bounding box
- */
 LIBSBML_EXTERN
 double
 BoundingBox_y(BoundingBox_t* bb)
 {
-    return bb->y();
+  if (bb == NULL) return std::numeric_limits<double>::quiet_NaN();
+  return bb->y();
 }
 
-/**
- * Returns the z offset of the bounding box
- */
 LIBSBML_EXTERN
 double
 BoundingBox_z(BoundingBox_t* bb)
 {
-    return bb->z();
+  if (bb == NULL) return std::numeric_limits<double>::quiet_NaN();
+  return bb->z();
 }
 
-/**
- * Returns the width of the bounding box
- */
 LIBSBML_EXTERN
 double
 BoundingBox_width(BoundingBox_t* bb)
 {
-    return bb->width();
+  if (bb == NULL) return std::numeric_limits<double>::quiet_NaN();
+  return bb->width();
 }
 
-/**
- * Returns the height of the bounding box
- */
 LIBSBML_EXTERN
 double
 BoundingBox_height(BoundingBox_t* bb)
 {
-    return bb->height();
+  if (bb == NULL) return std::numeric_limits<double>::quiet_NaN();
+  return bb->height();
 }
 
-/**
- * Returns the depth of the bounding box
- */
 LIBSBML_EXTERN
 double
 BoundingBox_depth(BoundingBox_t* bb)
 {
-    return bb->depth();
+  if (bb == NULL) return std::numeric_limits<double>::quiet_NaN();
+  return bb->depth();
 }
 
-/**
- * @return a (deep) copy of this Model.
- */
 LIBSBML_EXTERN
 BoundingBox_t *
 BoundingBox_clone (const BoundingBox_t *m)
 {
+  if (m == NULL) return NULL;
   return static_cast<BoundingBox*>( m->clone() );
 }
 
-/**
- * Returns non-zero if the id is set
- */
 LIBSBML_EXTERN
 int
 BoundingBox_isSetId (const BoundingBox_t *bb)
 {
+  if (bb == NULL) return (int)false;
   return static_cast <int> (bb->isSetId());
 }
 
-/**
- * Returns the id
- */
 LIBSBML_EXTERN
 const char *
 BoundingBox_getId (const BoundingBox_t *bb)
 {
+  if (bb == NULL) return NULL;
   return bb->isSetId() ? bb->getId().c_str() : NULL;
 }
 
-/**
- * Sets the id
- */
 LIBSBML_EXTERN
 int
 BoundingBox_setId (BoundingBox_t *bb, const char *sid)
 {
+  if (bb == NULL) return LIBSBML_INVALID_ATTRIBUTE_VALUE;
   return (sid == NULL) ? bb->setId("") : bb->setId(sid);
 }
 
-/**
- * Unsets the id
- */
 LIBSBML_EXTERN
 void
 BoundingBox_unsetId (BoundingBox_t *bb)
 {
+  if (bb == NULL) return;
   bb->unsetId();
 }
-
+/** @endcond */
 LIBSBML_CPP_NAMESPACE_END
 
 
